;**************************************************************************
;
;Die Benutzung der Software ist mit folgenden Bedingungen verbunden:
;
;1. Da ich alles kostenlos zur Verfgung stelle, gebe ich keinerlei Garantie
;   und bernehme auch keinerlei Haftung fr die Folgen der Benutzung.
;
;2. Die Software ist ausschlielich zur privaten Nutzung bestimmt. Ich
;   habe nicht geprft, ob bei gewerblicher Nutzung irgendwelche Patentrechte
;   verletzt werden oder sonstige rechtliche Einschrnkungen vorliegen.
;
;3. Jeder darf nderungen vornehmen, z.B. um die Funktion seinen Bedrfnissen
;   anzupassen oder zu erweitern. Ich wrde mich freuen, wenn ich weiterhin als
;   Co-Autor in den Unterlagen erscheine.
;
;4. Auch nach den nderungen sollen die Software weiterhin frei sein, d.h. 
;   kostenlos bleiben.
;
;!! Wer mit den Nutzungbedingungen nicht einverstanden ist, darf die Software 
;   nicht nutzen !!
;
; April 2009
; autor: Francesco Lanza
;

  list       p = 10F202

  include    P10F202.INC

  __CONFIG        _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC 
   
;==========================================================================
;
;       VariableS
 
; mem is from 0x08 to 0x18
Counter                 EQU     0x10		
TIMER                   EQU     0x11         
PPMSIG               	EQU     0x12		
PWMSIG               	EQU     0x13		
TEMP               	EQU     0x14		
CTH			EQU     0x15		

FIL1			EQU	0x08
FIL2			EQU	0x09
FIL3			EQU	0x0a
FIL4			EQU	0x0b
FIL5			EQU	0x0c

;GP0 - pin5 pwm input, 0.5ms rate
;GP1 - pin4 test output, the pulse here must be a multiple of pwm rate. tweak maxpwm.
;GP2 - pin3 ppm output
;GP3 - pin8 calib input, short to gnd at start for bl-esc max-min-ppm calibration

#define MAXPWM 	0xcf 		
#define PWMIN 	GPIO,0 		
#define TSTOUT 	GPIO,1 		
#define PPMOUT 	GPIO,2 		
#define CALIN 	GPIO,3 		

;=====================================    

        ORG	0       			;PIC's reset vector
    	MOVWF	OSCCAL              		;Calibrate the Int RC osc.
        GOTO	START                           ;Jump to START routine when boot.

;-------------------------------------

START   
	; timer0 has to generate 20ms clock
	; at 1mhz input tocs=0 psa=0 prescale=101     
        BCF	OSCCAL,FOSC4		; GP2 as I/O !
        MOVLW	B'10000101'	    	;->Config PIC options 
	OPTION	   		        
       
    	MOVLW	B'11111001'		; GP2 IS output.
	TRIS	GPIO			; bit 4-7 => Don't care.
        CLRF    GPIO			; Clear All output's.

	MOVLW	0x00
	MOVWF	PPMSIG			; ppm signal zero at start
	MOVWF	FIL1			; filt signal zero at start
	MOVWF	FIL2			; filt signal zero at start
	MOVWF	FIL3			; filt signal zero at start
	MOVWF	FIL4			; filt signal zero at start
	MOVWF	FIL5			; filt signal zero at start

docal:
	BTFSC	CALIN			; test for calib
	GOTO	loop

	MOVLW	MAXPWM	
	MOVWF	PPMSIG	
	CALL	ppmout
wcycal:
	; wait 16ms for next cycle
	MOVFW	TMR0
	BTFSS	STATUS,Z
	GOTO	wcycal

	CLRF	PPMSIG	
	GOTO	docal	

;-------------------------------------

loop:	
	CALL	ppmout

	; now count the pwm stuff  
	MOVLW	MAXPWM
	MOVWF	Counter	
	CLRF	PWMSIG		
	CLRF	TEMP	

	CLRF	TIMER
wt4lo:
	BTFSC	PWMIN
        GOTO    wt4hi                          
	DECFSZ  TIMER,F        
        GOTO    wt4lo                          
	CLRF	TIMER
wt4hi:
	BTFSS	PWMIN
        GOTO    wtend                          
	DECFSZ  TIMER,F        
        GOTO    wt4hi                          

wtend:	
	BSF	TSTOUT			; tst signal ON
pwmcnt:
	BTFSC	PWMIN
	INCF	PWMSIG,F
	BTFSS	PWMIN
	INCF	TEMP,F
	DECFSZ  Counter,F        
        GOTO    pwmcnt                          
	BCF	TSTOUT			; tst signal OFF 	

	; add 5 filter + actual value
	CLRF	TEMP
	MOVFW	PWMSIG
	ADDWF	FIL1,0
	BTFSC   STATUS,C
	INCF	TEMP,1
	ADDWF	FIL2,0
	BTFSC   STATUS,C
	INCF	TEMP,1
	ADDWF	FIL3,0
	BTFSC   STATUS,C
	INCF	TEMP,1
	;ADDWF	FIL4,0
	;BTFSC   STATUS,C
	;INCF	TEMP,1
	;ADDWF	FIL5,0		;too much power !
	;BTFSC   STATUS,C
	;INCF	TEMP,1


	;Divide trough 8
	MOVWF	PPMSIG
	BCF	STATUS,C
	RRF	TEMP,1
	RRF	PPMSIG,1
	RRF	TEMP,1
	RRF	PPMSIG,1
	RRF	TEMP,1
	RRF	PPMSIG,1

	;Shift and fill Filter 	
	MOVFW	FIL4
	MOVWF	FIL5
	MOVFW	FIL3
	MOVWF	FIL4
	MOVFW	FIL2
	MOVWF	FIL3
	MOVFW	FIL1
	MOVWF	FIL2
	MOVFW	PWMSIG
	MOVWF	FIL1

	;MOVFW	PWMSIG
	;MOVWF	PPMSIG

waitcycle:
	; wait 16ms for next cycle
	MOVFW	TMR0
	BTFSS	STATUS,Z
	GOTO	waitcycle

	GOTO	loop

;-------------------------------------

ppmout:
	; generate the ppm pulse, 1.1 to 1.9 ms long	
	MOVFW	PPMSIG			; delay
	MOVWF	TIMER			; 0.8 ms range
	INCF	TIMER,1
	MOVLW	0xde
	MOVWF	CTH			; Pause 1.1ms tot
	BSF	PPMOUT			; ppm signal ON
ppmhi1:
	NOP
	NOP
	DECFSZ  CTH,F        
        GOTO    ppmhi1                          
ppmhi2:
	NOP
	DECFSZ  TIMER,F        
        GOTO    ppmhi2                          
	BCF	PPMOUT			; ppm signal OFF 
	RETLW	0

;-------------------------------------

        END                          		;End of source code.


 
